<?php
namespace WC_Delivery_Time\Includes;

use WC_Delivery_Time\Includes\Helper\Model;

class WC_Delivery_Time_Show {

    protected static $_instance = null;
    private $model;

    public static function get_instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    public function __construct() {
        $this->model = Model::get_instance();

        add_action('woocommerce_checkout_process', array($this, 'woocommerce_checkout_process'));
        add_action('woocommerce_checkout_update_order_meta', array($this, 'woocommerce_checkout_update_order_meta'), 99);
        add_action('woocommerce_admin_order_data_after_billing_address', array($this, 'woocommerce_admin_order_data_after_billing_address'), 10, 1 );
        add_action('woocommerce_order_details_after_order_table_items', array($this, 'woocommerce_admin_order_data_after_billing_address'), 10, 1 );

        add_action( 'check_delivery_count', array($this, 'check_delivery_count_callback'), 10, 3 );

        $showLocation = wcdt_get_option('show_location');
        if(!isset($showLocation) or empty($showLocation)) $showLocation = 'woocommerce_review_order_before_payment';
        add_action($showLocation, array($this, 'delivery_time_field'), 99);

        add_action('woocommerce_checkout_update_order_review', [$this, 'woocommerce_checkout_update_order_review'], 10, 1);
        add_filter('woocommerce_update_order_review_fragments', [$this, 'woocommerce_update_order_review_fragments'], 10, 1);
        add_filter('delivery_time_show_condition', [$this, 'shipping_method_show_condition'], 10, 1);
    }

    public function shipping_method_show_condition($show) {

        $disabled_for_shipping_method = wcdt_get_option('disable_for_shipping_methods');
        if(!isset($disabled_for_shipping_method) or empty($disabled_for_shipping_method)) $disabled_for_shipping_method = [];
        else $disabled_for_shipping_method = array_values((array)$disabled_for_shipping_method);

        //@todo check alopeyk shipping method instance
        if(in_array('alopeyk_woocommerce_shipping_method', $disabled_for_shipping_method)) {
            $alopeyk_key = array_search ('alopeyk_woocommerce_shipping_method', $disabled_for_shipping_method);
            $disabled_for_shipping_method[$alopeyk_key] = 'alopeyk_woocommerce_shipping_method-motorbike';
        }

        $chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
        $chosen_shipping = $chosen_methods[0];
        $chosen_shipping = explode(':', $chosen_shipping);
        $chosen_shipping = $chosen_shipping[0];

        if(in_array($chosen_shipping, $disabled_for_shipping_method)) return false;
        return $show;
    }

    public function woocommerce_update_order_review_fragments($array) {

        ob_start();
        echo '<div class="wcdt-container">';
        self::show_delivery_time();
        echo '</div>';
        $container = ob_get_clean();
        $array['.wcdt-container'] = $container;
        return $array;
    }

    public function woocommerce_checkout_update_order_review($data) {

    }

    public static function get_settings($option = false) {
        return WC_Delivery_Time_Settings::get_option($option);
    }

    public static function time_date_i18n($format, $date) {
        return date_i18n( $format, strtotime( self::time_date($format, $date) ) );
    }
    public static function time_date($format, $date) {
        return date($format, strtotime(wcdt_date($date)));
    }

    public static function date_timestamp($date) {
        return strtotime( $date );
    }

    public static function convert_number($string, $input = 'fa') {
        if($input === 'en') {
            return strtr($string, array('0'=>'۰','1'=>'۱','2'=>'۲','3'=>'۳','4'=>'۴','5'=>'۵','6'=>'۶','7'=>'۷','8'=>'۸','9'=>'۹'));
        } else {
            return strtr($string, array('۰'=>'0', '۱'=>'1', '۲'=>'2', '۳'=>'3', '۴'=>'4', '۵'=>'5', '۶'=>'6', '۷'=>'7', '۸'=>'8', '۹'=>'9', '٠'=>'0', '١'=>'1', '٢'=>'2', '٣'=>'3', '٤'=>'4', '٥'=>'5', '٦'=>'6', '٧'=>'7', '٨'=>'8', '٩'=>'9'));
        }
    }

    public static function show_hour($hour) {
        return wcdt_render_setting_hour_range($hour);
    }

    public static function get_delivery_meta_item($order_id) {
        global $wpdb;
        $table = $wpdb->prefix.'woocommerce_delivery_time';
        $deliveryInfo = $wpdb->get_results("SELECT * FROM {$table} WHERE `delivery_orders` LIKE '%{$order_id}%' ", 'ARRAY_A');

        return $deliveryInfo;
    }

    public static function get_delivery_meta($date, $time) {
        global $wpdb;
        $table = $wpdb->prefix.'woocommerce_delivery_time';
        $deliveryInfo = $wpdb->get_results("SELECT * FROM {$table} WHERE `delivery_date` = {$date} AND `delivery_time` = '{$time}' ", 'ARRAY_A');

        return $deliveryInfo;
    }

    public static function always_show() {
        $always_show =  wcdt_get_option('always_show');
        if(isset($always_show) and !empty($always_show) and $always_show === 'yes') {
            return true;
        } else {
            return false;
        }
    }

    public static function show_off_dates() {
        $show_off_dates = wcdt_get_option('show_off_dates');
        if(isset($show_off_dates) and !empty($show_off_dates) and $show_off_dates === 'yes') {
            return true;
        } else {
            return false;
        }
    }

    public static function province_for_pw_changes($province) {
        $oldStates = ["AE", "AW", "AR", "IS", "AL", "IL", "BU", "TE", "CM", "KJ", "KV", "KS", "KZ", "ZA", "SM", "SB", "FA", "QZ", "QM", "KD", "KE", "BK", "KB", "GO", "GI", "LO", "MN", "MK", "HG", "HD", "YA"];
        $newStates = ["EAZ", "WAZ", "ADL", "ESF", "ABZ", "ILM", "BHR", "THR", "CHB", "SKH", "RKH", "NKH", "KHZ", "ZJN", "SMN", "SBN", "FRS", "GZN", "QHM", "KRD", "KRN", "KRH", "KBD", "GLS", "GIL", "LRS", "MZN", "MKZ", "HRZ", "HDN", "YZD"];


        if( (defined('PW_VERSION') and !PW_VERSION < '3.5.4') or (defined('WCDT_VERSION')) )
            return str_replace($oldStates, $newStates, $province);
        else
            return $province;
    }

    public static function delivery_time_field($checkout){
        echo '<div class="wcdt-container"></div>';
    }

    public function show_delivery_time() {
        $theme =  wcdt_get_option('theme', 'design');
        $theme_mobile =  wcdt_get_option('theme_mobile', 'design');

        $cart_items = \WC()->cart->cart_contents;
        $product_skip_days = 0;
        foreach($cart_items as $item) {
            $product_id = $item['product_id'];
            $variation_id = $item['variation_id'];

            if($variation_id === 0) {
                $pid = $product_id;
            } else {
                $pid = $variation_id;
            }


            $delivery_ready_days = get_post_meta( $pid, '_delivery_ready_in_days', true );
            if(isset($delivery_ready_days) and !empty($delivery_ready_days)) {
                if($product_skip_days < $delivery_ready_days) {
                    $product_skip_days = $delivery_ready_days;
                }
                continue;
            }


            $product_cats = get_the_terms( $product_id, 'product_cat' );
            if(isset($product_cats) and !empty($product_cats)) {
                foreach ($product_cats as $key => $term) {
                    $term_id = $term->term_id;
                    $cat_ready_days = get_term_meta($term_id, '_wcdt_cat_ready_days', true);
                    if(isset($cat_ready_days) and !empty($cat_ready_days)) {
                        if($product_skip_days < $cat_ready_days) {
                            $product_skip_days = $cat_ready_days;
                        }
                    }

                }
            }

        }


        $location_condition = wcdt_check_location_condition_with_fastsending();
        $always_show_condition = self::always_show();
        $delivery_time_show_condition = (!$location_condition and !$always_show_condition) ? false : true;
        $delivery_time_show_condition = apply_filters('delivery_time_show_condition', $delivery_time_show_condition, $location_condition, $always_show_condition);
        if( !$delivery_time_show_condition ) {
            echo '';
            return;
        }

        $daysToShowNumber =  wcdt_get_option('number_show_days');
        $dayStartFrom =  wcdt_get_option('days_start_from');
        $dayStartFrom = ($product_skip_days < $dayStartFrom) ? $dayStartFrom : $product_skip_days;
        $offWeekDays =  wcdt_get_option('week_days_without_delivery');
        $offDates =  explode(',', self::convert_number(wcdt_get_option('dates_without_delivery')));
        $deliveryHours = wcdt_get_option('days_delivery_hours');
        $current_hour = self::time_date('H', 0 . ' day');
        $defaultDeactiveHour = wcdt_get_option('deactive_hours_range_default');
        $defaultDeactiveHour = (isset($defaultDeactiveHour) and !empty($defaultDeactiveHour)) ? $defaultDeactiveHour : 0;

        if(!isset($theme) or empty($theme)) $theme = 'classic';
        if(wcdt_is_mobile() and isset($theme_mobile) and !empty($theme_mobile) and $theme_mobile != 'default') $theme = $theme_mobile;
        include WCDT_PATH . 'templates/' . $theme . '.php';
    }

    public static function date_format() {
        $format = wcdt_get_option('format', 'design');
        return !empty($format) ? $format : get_option('date_format');
    }

    public static function woocommerce_checkout_process(){

        $location_condition = wcdt_check_location_condition_with_fastsending();
        $always_show_condition = self::always_show();
        $delivery_time_show_condition = (!$location_condition and !$always_show_condition) ? false : true;
        $delivery_time_show_condition = apply_filters('delivery_time_show_condition', $delivery_time_show_condition, $location_condition, $always_show_condition);
        if( $delivery_time_show_condition ) {
            if(!$_POST[ 'delivery_date' ]){
                $wcdt_error = wcdt_get_option('not_selected_error', 'text');
                if(isset($wcdt_error) and !empty($wcdt_error)):
                    wc_add_notice($wcdt_error, 'error');
                else:
                    wc_add_notice(__('Please select delivery time.', 'wcdt'), 'error');
                endif;
            }
        }
    }

    public static function different_hours($dateFrom, $dateTo) {
        $seconds = strtotime($dateTo) - strtotime($dateFrom);

        $days    = floor($seconds / 86400);
        $hours   = floor(($seconds - ($days * 86400)) / 3600);
        return $hours + $days*24;
    }

    public function woocommerce_checkout_update_order_meta($order_id){
        if(isset($_POST[ 'delivery_date' ]) and !empty($_POST[ 'delivery_date' ])) {
            $data = explode('-', $_POST[ 'delivery_date' ]);
            $date = $data[0];
            $time = $data[1].'-'.$data[2];
            $this->model->update_delivery_meta($date, $time, $order_id, 1);
            $this->model->update_delivery_volume($date, $time, $order_id);

            $delivery_date = $date;
            $delivery_hour = $time;
            $result = WC_Delivery_Time_Show::convert_number(WC_Delivery_Time_Show::get_date_i18n($delivery_date)) . ' - ' . $delivery_hour;
            update_post_meta($order_id, '_delivery_time', $result);

            wp_schedule_single_event( time() + 15*60, 'check_delivery_count', array( $order_id ) );
        }
    }

    public function check_delivery_count_callback( $order_id ) {
        $order_status = get_post_status( $order_id );
        $to_remove_statuses = wcdt_order_statuses_to_reduce_volume();

        if(in_array($order_status, $to_remove_statuses)) {
            $delivery_info = self::get_delivery_meta_item( $order_id );
            if(isset($delivery_info) && !empty($delivery_info)){
                $delivery_date = $delivery_info[0]['delivery_date'];
                $delivery_hour = $delivery_info[0]['delivery_time'];
                $this->model->reduce_delivery_volume($delivery_date, $delivery_hour, $order_id);
            }
        }

    }

    public static function get_date($timestamp) {
        return get_date_from_gmt(date('Y/m/d H:i:s', $timestamp), 'Y/m/d');
    }

    public static function get_date_i18n($timestamp) {
        return date_i18n('Y/m/d', strtotime(self::get_date($timestamp)));
    }

    public static function woocommerce_admin_order_data_after_billing_address($order){
        $order_id = $order->get_id();
        $delivery_info = self::get_delivery_meta_item( $order_id );
        if(isset($delivery_info) && !empty($delivery_info)){
            $delivery_date = $delivery_info[0]['delivery_date'];
            $delivery_hour = $delivery_info[0]['delivery_time'];
            echo '<div class="show-delivery-time"><strong>'. __('delivery date and time: ', 'wcdt') .'</strong><span>' . __('date: ', 'wcdt') . self::convert_number(self::get_date_i18n($delivery_date)) . ' - ' . __('time: ', 'wcdt') .$delivery_hour.'</span></div>';
        }
    }

}